@@ -13,7 +13,10 @@ import numpy as np
 import zstandard as zstd
 
 
-__version__ = '0.2.3'
+__version__ = '0.3.0'
+
+
+_logger = logging.getLogger(__name__)
 
 
 @contextlib.contextmanager
@@ -71,7 +74,7 @@ class Module(metaclass=ABCMeta):
 
     @property
     def hash(self):
-        return hashlib.sha1(inspect.getsource(self.execute).encode('utf-8'))
+        return hashlib.sha1(self.execute.__code__.co_code)
 
     @abstractmethod
     def execute(self):
@@ -98,6 +101,7 @@ class Context:
         self._product_dir = product_dir
         self._modules = set()
         self._producer = {}
+        _logger.info(f'Using "{self._product_dir}" as a product repository')
 
     def register_modules(self, *modules):
         for mod in modules:
@@ -105,7 +109,10 @@ class Context:
                 warnings.warn(f'Module "{mod.name}" is not an instance of Module', RuntimeWarning)
             self._modules.add(mod)
             for name in mod.output_names:
+                if name in self._producer:
+                    warnings.warn(f'Producer for "{name}" is already registered')
                 self._producer[name] = mod
+            _logger.info(f'Module "{mod.name}" has been registered')
 
     def module(self, input, output):
         decorator = module(input, output)
@@ -118,8 +125,8 @@ class Context:
     def _get_path(self, name, run_hash):
         return os.path.join(
             self._product_dir,
-            name,
-            run_hash)
+            run_hash,
+            name)
 
     def _has(self, name, run_hash):
         path = self._get_path(name, run_hash)
@@ -159,8 +166,10 @@ class Context:
                 if os.path.exists(path):
                     os.remove(path)
                 raise e
+        _logger.info(f'Product has been saved as "{path}"')
 
     def run(self, module, condition=None):
+        _logger.info(f'Running module "{module.name}"')
         if condition is None:
             condition = {}
         # Run upstream modules
@@ -182,8 +191,10 @@ class Context:
         run_hash = h.hexdigest()
         # Execute the module if needed
         if all(self._has(name, run_hash) for name in module.output_names):
+            _logger.info(f'Found cached products of module "{module.name}", skipping execution')
             products = tuple(self._get(name, run_hash) for name in module.output_names)
         else:
+            _logger.info(f'Executing module "{module.name}"')
             products = module.execute(*args, **kwargs)
             if len(module.output_names) == 1:
                 products = (products, )
